@@ -73,7 +73,7 @@ int Message::_commSize = 1;
 int Message::_isCommWorld = 1; // is the communicator set to WORLD (==1) or SELF (!=1)
 int Message::_errorCount = 0;
 int Message::_lastPETScError = 0;
-bool Message::_exitOnError = false;
+int Message::_exitOnError = 0;
 bool Message::_operatingInTimeLoopAdaptive = false;
 int Message::_verbosity = 5;
 int Message::_progressMeterStep = 10;
@@ -154,9 +154,8 @@ void Message::Initialize(int argc, char **argv)
 void Message::Finalize()
 {
 #if defined(HAVE_PETSC)
-  // if: to avoid a 'PETSC ERROR' message when calling MPI_Finalize()
-  // (not coherent with prior MPI_Init(&argc, &argv)... but do nothing when _commSize == 1)
-  if(_commSize >= 1){
+  // don't call MPI_*() if _commSize == 1: newer PETSc does like it
+  if(_commSize > 1){
     int initialized, finalized;
     MPI_Initialized(&initialized);
     MPI_Finalized(&finalized);
@@ -293,10 +292,12 @@ void Message::Error(const char *fmt, ...)
     fflush(stderr);
   }
 
-  if(_exitOnError){
-    // Error() should normally not exit; use Fatal() for that
+  if(_exitOnError == 1){
     Exit(1);
   }
+  else if(_exitOnError == 2){
+    throw "GetDP error";
+  }
 }
 
 void Message::Warning(const char *fmt, ...)
@@ -759,7 +760,7 @@ public:
   void sendInfo(const std::string &msg){ Msg::Info("%s", msg.c_str()); }
   void sendWarning(const std::string &msg){ Msg::Warning("%s", msg.c_str()); }
   void sendError(const std::string &msg){ Msg::Error("%s", msg.c_str()); }
-  void sendProgress(const std::string &msg){ Msg::ProgressMeter(0, 0, true, msg.c_str()); }
+  void sendProgress(const std::string &msg){ }
 #endif
 };
 
@@ -883,22 +884,22 @@ void Message::AddOnelabStringChoice(std::string name, std::string kind,
   }
 }
 
-void Message::SetOnelabNumber(std::string name, double val, bool visible)
+void Message::SetOnelabNumber(std::string name, double val)
 {
   if(_onelabClient){
     std::vector<onelab::number> numbers;
+    // get if first so we can keep its options
     _onelabClient->get(numbers, name);
     if(numbers.empty()){
       numbers.resize(1);
       numbers[0].setName(name);
     }
     numbers[0].setValue(val);
-    numbers[0].setVisible(visible);
     _onelabClient->set(numbers[0]);
   }
 }
 
-void Message::SetOnelabString(std::string name, std::string val, bool visible)
+void Message::SetOnelabString(std::string name, std::string val)
 {
   if(_onelabClient){
     std::vector<onelab::string> strings;
@@ -908,7 +909,6 @@ void Message::SetOnelabString(std::string name, std::string val, bool visible)
       strings[0].setName(name);
     }
     strings[0].setValue(val);
-    strings[0].setVisible(visible);
     _onelabClient->set(strings[0]);
   }
 }
