@@ -16,20 +16,20 @@
 
 #define WrongNumEntity   Message::Error("Wrong Edge number in 'BF_Node_2E'")
 
-void BF_Node_2E(struct Element * Element, int NumEntity, 
+void BF_Node_2E(struct Element * Element, int NumEntity,
 		double u, double v, double w,  double *s)
 {
   switch (Element->Type) {
-  case LINE :
-  case LINE_2 :
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
     switch(NumEntity) {
     case 1  : *s = 0.25 * (1.-u) * (1.+u) ; break ;
     default : WrongNumEntity ;
     }
     break ;
 
-  case TRIANGLE :
-  case TRIANGLE_2 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
     switch(NumEntity) {
     case 1  : *s = (1.-u-v) * u ; break ;
     case 2  : *s = (1.-u-v) * v ; break ;
@@ -38,7 +38,8 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case QUADRANGLE :
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     switch(NumEntity) {
     case 1  : *s = 0.0625 * (1.-u*u) * (1.-v)*(1.-v) ; break ;
     case 2  : *s = 0.0625 * (1.-u)*(1.-u) * (1.-v*v) ; break ;
@@ -48,7 +49,8 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case TETRAHEDRON :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     switch(NumEntity) {
     case 1  : *s = (1.-u-v-w) * u ; break ;
     case 2  : *s = (1.-u-v-w) * v ; break ;
@@ -60,7 +62,8 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     switch(NumEntity) {
     case 1  : *s = 0.015625 * (1.-u) * (1.-v) * (1.-w) * (1.+u) * (1.-v) * (1.-w) ; break ;
     case 2  : *s = 0.015625 * (1.-u) * (1.-v) * (1.-w) * (1.-u) * (1.+v) * (1.-w) ; break ;
@@ -78,8 +81,10 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PRISM : // FIXME: not tested!
-    switch(NumEntity) { 
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+    // FIXME: not tested!
+    switch(NumEntity) {
     case 1  : *s = 0.25 * (1.-u-v) * (1.-w) *     u    * (1.-w) ; break ;
     case 2  : *s = 0.25 * (1.-u-v) * (1.-w) *       v  * (1.-w) ; break ;
     case 3  : *s = 0.25 * (1.-u-v) * (1.-w) * (1.-u-v) * (1.+w) ; break ;
@@ -93,7 +98,8 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PYRAMID :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     switch(NumEntity) {
     default : Message::Error("BF_Node_2E not ready for PYRAMID");
     }
@@ -114,24 +120,27 @@ void BF_Node_2E(struct Element * Element, int NumEntity,
 
 #define WrongNumEntity   Message::Error("Wrong Face number in 'BF_Node_2F'")
 
-void BF_Node_2F(struct Element * Element, int NumEntity, 
+void BF_Node_2F(struct Element * Element, int NumEntity,
 		double u, double v, double w,  double *s)
 {
   switch (Element->Type) {
 
-  case LINE :
-  case LINE_2 :
-  case TRIANGLE :
-  case TRIANGLE_2 :
-  case TETRAHEDRON :
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     Message::Error("BF_Node_2F cannot be associated with this type of element");
     break;
-    
-  case QUADRANGLE :
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     *s = 0.0625 * (1.-u) * (1.-v) * (1.+u) * (1.+v) ;
     break ;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     switch(NumEntity) {
     case 1  : *s = 0.015625 * (1.-u) * (1.-v) * (1.-w) * (1.+u) * (1.-v) * (1.+w) ; break ;
     case 2  : *s = 0.015625 * (1.-u) * (1.-v) * (1.-w) * (1.+u) * (1.+v) * (1.-w) ; break ;
@@ -143,13 +152,13 @@ void BF_Node_2F(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PRISM :
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
     switch(NumEntity) {
     default : Message::Error("BF_Node_2F not ready for PRISM");
-      // cannot do this yet in getdp, as dofs should only be
-      // associated with quad-faces: if really necessary we could
-      // implement actual 15 and 18-node prisms
-      /* 
+      // cannot do this, as dofs should only be associated with quad-faces; use
+      // 15 and 18-node prisms instead, i.e. through BF_Node (which calls Gmsh).
+      /*
     case 1  : *s = 0.25 * (1.-u-v) * (1.-w) *     u    * (1.+w) ; break ;
     case 3  : *s = 0.25 * (1.-u-v) * (1.-w) *       v  * (1.+w) ; break ;
     case 4  : *s = 0.25 *     u    * (1.-w) *       v  * (1.+w) ; break ;
@@ -158,7 +167,8 @@ void BF_Node_2F(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PYRAMID :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     switch(NumEntity) {
     default : Message::Error("BF_Node_2F not ready for PYRAMID");
     }
@@ -177,23 +187,28 @@ void BF_Node_2F(struct Element * Element, int NumEntity,
 /*  Volume  */
 /* -------- */
 
-void BF_Node_2V(struct Element * Element, int NumEntity, 
+void BF_Node_2V(struct Element * Element, int NumEntity,
 		double u, double v, double w,  double *s)
 {
   switch (Element->Type) {
 
-  case LINE :
-  case LINE_2 :
-  case TRIANGLE :
-  case TRIANGLE_2 :
-  case QUADRANGLE :
-  case TETRAHEDRON :
-  case PRISM :
-  case PYRAMID :
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     Message::Error("BF_Node_2V cannot be associated with this type of element");
     break;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     *s = 0.015625 * (1.-u) * (1.-v) * (1.-w) * (1.+u) * (1.+v) * (1.+w) ;
     break ;
 
@@ -201,7 +216,7 @@ void BF_Node_2V(struct Element * Element, int NumEntity,
     Message::Error("Unknown type of Element in BF_Node_2V");
     break ;
   }
- 
+
 }
 
 
@@ -215,20 +230,20 @@ void BF_Node_2V(struct Element * Element, int NumEntity,
 
 #define WrongNumEntity   Message::Error("Wrong Edge number in 'BF_GradNode_2E'")
 
-void  BF_GradNode_2E(struct Element * Element, int NumEntity, 
+void  BF_GradNode_2E(struct Element * Element, int NumEntity,
 		     double u, double v, double w,  double s[])
 {
   switch (Element->Type) {
-  case LINE :
-  case LINE_2 :
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
     switch(NumEntity) {
     case 1  : s[0] = -0.5*u ; s[1] = 0. ; s[2] = 0. ; break ;
     default : WrongNumEntity ;
     }
     break ;
 
-  case TRIANGLE :
-  case TRIANGLE_2 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
     switch(NumEntity) {
     case 1  : s[0] = 1.-2.*u-v ; s[1] = -u        ; s[2] = 0. ; break ;
     case 2  : s[0] = -v        ; s[1] = 1.-u-2.*v ; s[2] = 0. ; break ;
@@ -237,10 +252,11 @@ void  BF_GradNode_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case QUADRANGLE :
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     switch(NumEntity) {
     case 1  : s[0] = 0.0625 * (-2.*u) * (1.-v)*(1.-v) ;
-              s[1] = 0.0625 * (1.-u*u) * (-2.)*(1.-v) ; 
+              s[1] = 0.0625 * (1.-u*u) * (-2.)*(1.-v) ;
               s[2] = 0. ; break ;
     case 2  : s[0] = 0.0625 * (-2.)*(1.-u) * (1.-v*v) ;
               s[1] = 0.0625 * (1.-u)*(1.-u) * (-2.*v) ;
@@ -255,7 +271,8 @@ void  BF_GradNode_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case TETRAHEDRON :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     switch(NumEntity) {
     case 1  : s[0] =  1.-2.*u-v-w ; s[1] = -u           ; s[2] = -u           ; break ;
     case 2  : s[0] = -v           ; s[1] =  1.-u-2.*v-w ; s[2] = -v           ; break ;
@@ -267,105 +284,109 @@ void  BF_GradNode_2E(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     switch(NumEntity) {
-    case 1   : s[0] = 0.015625 * (-2.*u) * (1.-v) * (1.-w) * (1.-v) * (1.-w) ; 
-               s[1] = 0.015625 * (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.-w) ; 
+    case 1   : s[0] = 0.015625 * (-2.*u) * (1.-v) * (1.-w) * (1.-v) * (1.-w) ;
+               s[1] = 0.015625 * (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.-w) ;
                s[2] = 0.015625 * (1.-u) * (1.-v) * (-2.)*(1.-w) * (1.+u) * (1.-v) ; break ;
 
-    case 2   : s[0] = 0.015625 * (-2.)*(1.-u) * (1.-v) * (1.-w) * (1.+v) * (1.-w) ; 
-               s[1] = 0.015625 * (1.-u) * (-2.*v) * (1.-w) * (1.-u) * (1.-w) ; 
+    case 2   : s[0] = 0.015625 * (-2.)*(1.-u) * (1.-v) * (1.-w) * (1.+v) * (1.-w) ;
+               s[1] = 0.015625 * (1.-u) * (-2.*v) * (1.-w) * (1.-u) * (1.-w) ;
 	       s[2] = 0.015625 * (1.-u) * (1.-v) * (-2.)*(1.-w) * (1.-u) * (1.+v) ; break ;
 
-    case 3   : s[0] = 0.015625 *  (-2.)*(1.-u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.-u) * (1.+w) ; 
+    case 3   : s[0] = 0.015625 *  (-2.)*(1.-u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.-u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.-u) * (1.-v) * (-2.*w) * (1.-u) * (1.-v) ; break ;
 
-    case 4   : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.-w) * (1.+v) * (1.-w) ; 
-               s[1] = 0.015625 *  (1.+u) * (-2.*v) * (1.-w) * (1.+u) * (1.-w) ; 
+    case 4   : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.-w) * (1.+v) * (1.-w) ;
+               s[1] = 0.015625 *  (1.+u) * (-2.*v) * (1.-w) * (1.+u) * (1.-w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.-v) * (-2.)*(1.-w) * (1.+u) * (1.+v) ; break ;
 
-    case 5   : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.+u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.+w) ; 
+    case 5   : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.+u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.-v) * (-2.*w) * (1.+u) * (1.-v) ; break ;
- 
-    case 6   : s[0] = 0.015625 *  (-2.*u) * (1.+v) * (1.-w) * (1.+v) * (1.-w) ; 
-               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.-w) * (1.-u) * (1.-w) ; 
+
+    case 6   : s[0] = 0.015625 *  (-2.*u) * (1.+v) * (1.-w) * (1.+v) * (1.-w) ;
+               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.-w) * (1.-u) * (1.-w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.+v) * (-2.)*(1.-w) * (1.-u) * (1.+v) ; break ;
 
-    case 7   : s[0] = 0.015625 *  2.*(1.+u) * (1.+v) * (1.-w) * (1.+v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.-w) * (1.+u) * (1.+w) ; 
+    case 7   : s[0] = 0.015625 *  2.*(1.+u) * (1.+v) * (1.-w) * (1.+v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.-w) * (1.+u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.+v) * (-2.*w) * (1.+u) * (1.+v) ; break ;
 
-    case 8   : s[0] = 0.015625 *  (-2.)*(1.-u) * (1.+v) * (1.-w) * (1.+v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.-u) * 2.*(1.+v) * (1.-w) * (1.-u) * (1.+w) ; 
+    case 8   : s[0] = 0.015625 *  (-2.)*(1.-u) * (1.+v) * (1.-w) * (1.+v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.-u) * 2.*(1.+v) * (1.-w) * (1.-u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.-u) * (1.+v) * (-2.*w) * (1.-u) * (1.+v) ; break ;
 
-    case 9   : s[0] = 0.015625 *  (-2.*u) * (1.-v) * (1.+w) * (1.-v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.+w) * (1.+u) * (1.+w) ; 
+    case 9   : s[0] = 0.015625 *  (-2.*u) * (1.-v) * (1.+w) * (1.-v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.+w) * (1.+u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.-u) * (1.-v) * 2.*(1.+w) * (1.+u) * (1.-v) ; break ;
 
     case 10  : s[0] = 0.015625 *  (-2.)*(1.-u) * (1.-v) * (1.+w) * (1.+v) * (1.+w) ;
-               s[1] = 0.015625 *  (1.-u) * (-2.*v) * (1.+w) * (1.-u) * (1.+w) ; 
+               s[1] = 0.015625 *  (1.-u) * (-2.*v) * (1.+w) * (1.-u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.-u) * (1.-v) * 2.*(1.+w) * (1.-u) * (1.+v) ; break ;
 
-    case 11  : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.+w) * (1.+v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.+u) * (-2.*v) * (1.+w) * (1.+u) * (1.+w) ; 
+    case 11  : s[0] = 0.015625 *  2.*(1.+u) * (1.-v) * (1.+w) * (1.+v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.+u) * (-2.*v) * (1.+w) * (1.+u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.-v) * 2.*(1.+w) * (1.+u) * (1.+v) ; break ;
 
-    case 12  : s[0] = 0.015625 *  (-2.*u) * (1.+v) * (1.+w) * (1.+v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.+w) * (1.-u) * (1.+w) ; 
+    case 12  : s[0] = 0.015625 *  (-2.*u) * (1.+v) * (1.+w) * (1.+v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.+u) * 2.*(1.+v) * (1.+w) * (1.-u) * (1.+w) ;
 	       s[2] = 0.015625 *  (1.+u) * (1.+v) * 2.*(1.+w) * (1.-u) * (1.+v) ; break ;
-	       
+
     default : WrongNumEntity ;
     }
     break ;
 
-  case PRISM :  // FIXME: not tested!
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+    // FIXME: not tested!
     switch(NumEntity) {
-    case 1   : s[0] = 0.25 * (1.-2.*u-v) * (1.-w) * (1.-w) ; 
-               s[1] = 0.25 * (-u) * (1.-w) * (1.-w) ; 
+    case 1   : s[0] = 0.25 * (1.-2.*u-v) * (1.-w) * (1.-w) ;
+               s[1] = 0.25 * (-u) * (1.-w) * (1.-w) ;
                s[2] = 0.25 * (u-u*u-v*u) * (-2.)*(1.-w) ; break ;
 
-    case 2   : s[0] = 0.25 * (-v) * (1.-w) * (1.-w) ; 
-               s[1] = 0.25 * (1-u-2*v) * (1.-w) * (1.-w) ; 
+    case 2   : s[0] = 0.25 * (-v) * (1.-w) * (1.-w) ;
+               s[1] = 0.25 * (1-u-2*v) * (1.-w) * (1.-w) ;
                s[2] = 0.25 * (v-u*v-v*v) * (-2.) * (1.-w) ; break ;
 
-    case 3   : s[0] = 0.25 * (-2. + 2.*u + 2.*v) * (1.-w) * (1.+w) ; 
-               s[1] = 0.25 * (-2. + 2.*u + 2.*v) * (1.-w) * (1.+w) ; 
+    case 3   : s[0] = 0.25 * (-2. + 2.*u + 2.*v) * (1.-w) * (1.+w) ;
+               s[1] = 0.25 * (-2. + 2.*u + 2.*v) * (1.-w) * (1.+w) ;
                s[2] = 0.25 * (1.- 2.*u - 2.*v + u*u + 2.*u*v + v*v) * (-2.*w) ; break ;
 
-    case 4   : s[0] = 0.25 * (1.-w) * v * (1.-w) ; 
-               s[1] = 0.25 * u * (1.-w) * (1.-w) ; 
+    case 4   : s[0] = 0.25 * (1.-w) * v * (1.-w) ;
+               s[1] = 0.25 * u * (1.-w) * (1.-w) ;
                s[2] = 0.25 * u * (-2.) * (1.-w) * v ; break ;
 
-    case 5   : s[0] = 0.25 * 2.*u * (1.-w) * (1.+w) ; 
-               s[1] = 0. ; 
+    case 5   : s[0] = 0.25 * 2.*u * (1.-w) * (1.+w) ;
+               s[1] = 0. ;
                s[2] = 0.25 * u*u * (-2.*w) ; break ;
 
-    case 6   : s[0] = 0. ; 
-               s[1] = 0.25 * 2.*v  * (1.-w) * (1.+w) ; 
+    case 6   : s[0] = 0. ;
+               s[1] = 0.25 * 2.*v  * (1.-w) * (1.+w) ;
                s[2] = 0.25 * v*v  * (-2.*w) ; break ;
 
-    case 7   : s[0] = 0.25 * (1.-2.*u-v) * (1.+w) * (1.+w) ; 
-               s[1] = 0.25 * (-u) * (1.+w) * (1.+w) ; 
+    case 7   : s[0] = 0.25 * (1.-2.*u-v) * (1.+w) * (1.+w) ;
+               s[1] = 0.25 * (-u) * (1.+w) * (1.+w) ;
                s[2] = 0.25 * (u-u*u-u*v) * 2.*(1.+w) ; break ;
 
-    case 8   : s[0] = 0.25 * (-v) * (1.+w) * (1.+w) ; 
-               s[1] = 0.25 * (1.-u-2.*v) * (1.+w) * (1.+w) ; 
+    case 8   : s[0] = 0.25 * (-v) * (1.+w) * (1.+w) ;
+               s[1] = 0.25 * (1.-u-2.*v) * (1.+w) * (1.+w) ;
                s[2] = 0.25 * (v-u*v-v*v) * 2.*(1.+w) ; break ;
 
-    case 9   : s[0] = 0.25 * (1.+w) * v * (1.+w) ; 
-               s[1] = 0.25 * u * (1.+w) * (1.+w) ; 
+    case 9   : s[0] = 0.25 * (1.+w) * v * (1.+w) ;
+               s[1] = 0.25 * u * (1.+w) * (1.+w) ;
                s[2] = 0.25 * u * 2.*(1.+w) * v ; break ;
 
     default : WrongNumEntity ;
     }
     break ;
 
-  case PYRAMID :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     switch(NumEntity) {
-    default : Message::Error("BF_GradNode_2E not ready for PYRAMID"); 
+    default : Message::Error("BF_GradNode_2E not ready for PYRAMID");
     }
     break ;
 
@@ -384,29 +405,32 @@ void  BF_GradNode_2E(struct Element * Element, int NumEntity,
 
 #define WrongNumEntity   Message::Error("Wrong Face number in 'BF_GradNode_2F'")
 
-void BF_GradNode_2F(struct Element * Element, int NumEntity, 
+void BF_GradNode_2F(struct Element * Element, int NumEntity,
 		    double u, double v, double w,  double s[])
 {
   switch (Element->Type) {
 
-  case LINE :
-  case LINE_2 :
-  case TRIANGLE :
-  case TRIANGLE_2 :
-  case TETRAHEDRON :    
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     Message::Error("BF_GradNode_2F cannot be associated with this type of element");
     break;
-  
-  case QUADRANGLE :
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     s[0] = 0.0625 * (-2.*u) * (1.-v) * (1.+v) ;
     s[1] = 0.0625 * (1.-u) * (-2.*v) * (1.+u) ;
     s[2] = 0. ;
     break ;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     switch(NumEntity) {
-    case 1   : s[0] = 0.015625 *  (-2.*u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ; 
-               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.+w) ; 
+    case 1   : s[0] = 0.015625 *  (-2.*u) * (1.-v) * (1.-w) * (1.-v) * (1.+w) ;
+               s[1] = 0.015625 *  (1.-u) * (-2.)*(1.-v) * (1.-w) * (1.+u) * (1.+w) ;
                s[2] = 0.015625 *  (1.-u) * (1.-v) * (-2.*w) * (1.+u) * (1.-v) ; break ;
 
     case 2   : s[0] = 0.015625 *  (-2.*u) * (1.-v) * (1.-w) * (1.+v) * (1.-w) ;
@@ -432,15 +456,16 @@ void BF_GradNode_2F(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PRISM :
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
     switch(NumEntity) {
     default : Message::Error("BF_GradNode_2F not ready for PRISM");
       // cannot do this yet in getdp, as dofs should only be
       // associated with quad-faces: if really necessary we could
       // implement actual 15 and 18-node prisms
-      /* 
-    case 1   : s[0] = 0.25 * (1.-2.*u-v) * (1.-w) * (1.+w) ; 
-               s[1] = 0.25 * (-u) * (1.-w) * (1.+w); 
+      /*
+    case 1   : s[0] = 0.25 * (1.-2.*u-v) * (1.-w) * (1.+w) ;
+               s[1] = 0.25 * (-u) * (1.-w) * (1.+w);
                s[2] = 0.25 * (u-u*u-u*v) * (-2.*w); break ;
 
     case 3   : s[0] = 0.25 * (-v) * (1.-w) * (1.+w) ;
@@ -454,9 +479,10 @@ void BF_GradNode_2F(struct Element * Element, int NumEntity,
     }
     break ;
 
-  case PYRAMID :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     switch(NumEntity) {
-    default : Message::Error("BF_GradNode_2F not ready for PYRAMID"); 
+    default : Message::Error("BF_GradNode_2F not ready for PYRAMID");
     }
     break ;
 
@@ -473,23 +499,28 @@ void BF_GradNode_2F(struct Element * Element, int NumEntity,
 /*  Volume  */
 /* -------- */
 
-void BF_GradNode_2V(struct Element * Element, int NumEntity, 
+void BF_GradNode_2V(struct Element * Element, int NumEntity,
 		    double u, double v, double w,  double s[])
 {
   switch (Element->Type) {
 
-  case LINE :
-  case LINE_2 :
-  case TRIANGLE :
-  case TRIANGLE_2 :
-  case QUADRANGLE :
-  case TETRAHEDRON :
-  case PRISM :
-  case PYRAMID :
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     Message::Error("BF_GradNode_2V cannot be associated with this type of element");
     break;
 
-  case HEXAHEDRON :
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     s[0] = 0.015625 * (-2.*u) * (1.-v) * (1.-w) * (1.+v) * (1.+w) ;
     s[1] = 0.015625 * (1.-u) * (-2.*v) * (1.-w) * (1.+u) * (1.+w) ;
     s[2] = 0.015625 * (1.-u) * (1.-v) * (-2.*w) * (1.+u) * (1.+v) ;
@@ -501,4 +532,3 @@ void BF_GradNode_2V(struct Element * Element, int NumEntity,
   }
 
 }
-
