@@ -56,7 +56,7 @@
 #define MAX_STACK_SIZE0     8
 #define MAX_STACK_SIZE     20
 #elif defined(HAVE_MULTIHARMONIC)
-#define NBR_MAX_HARMONIC   40
+#define NBR_MAX_HARMONIC   200
 #define MAX_STACK_SIZE0     2
 #define MAX_STACK_SIZE     40
 #else
@@ -217,7 +217,7 @@ struct ConstraintPerRegion {
     struct { int  Node1, Node2;    } Network;
     struct {
       int  RegionRefIndex, SubRegionRefIndex;
-      int  FilterIndex, CoefIndex, FunctionIndex;
+      int  FilterIndex, CoefIndex, FunctionIndex, FunctionRefIndex;
       int  FilterIndex2, CoefIndex2, FunctionIndex2;
       double ToleranceFactor;
     } Link;
@@ -959,7 +959,7 @@ struct Operation {
       char *String;
     } Error;
     struct {
-      char *FileName;
+      char *FileName, *RunTimeVar;
       int ViewTag;
     } GmshRead;
     struct {
@@ -1448,6 +1448,7 @@ struct PostOperation {
   bool     ResampleTime;
   List_T  *TimeValue_L, *TimeImagValue_L;
   int      LastTimeStepOnly, OverrideTimeStepValue, NoMesh, CatFile;
+  int      AppendTimeStepToFileName;
 };
 
 struct PostSubOperation {
@@ -1646,7 +1647,7 @@ struct CurrentData {
   double  RelaxFac, NbrTestedFac, SolveJacAdaptFailed; //+++
 
   // Iterative linear system solvers
-  double  KSPIterations, KSPIteration, KSPResidual;
+  double  KSPIterations, KSPIteration, KSPResidual, KSPSystemSize;
 };
 
 /* ------------------------------------------------------------------------ */
@@ -1729,23 +1730,27 @@ struct Element {
 #define HEXAHEDRON_2     (1<<12)
 #define PRISM_2          (1<<13)
 #define PYRAMID_2        (1<<14)
-#define QUADRANGLE_2_8N  (1<<16)
 
-#define LINE_3           (1<<17)
-#define TRIANGLE_3       (1<<18)
-#define QUADRANGLE_3     (1<<19)
-#define TETRAHEDRON_3    (1<<20)
-#define HEXAHEDRON_3     (1<<21)
-#define PRISM_3          (1<<22)
-#define PYRAMID_3        (1<<23)
+#define QUADRANGLE_2_8N  (1<<15)
+#define HEXAHEDRON_2_20N (1<<16)
+#define PRISM_2_15N      (1<<17)
+#define PYRAMID_2_13N    (1<<18)
 
-#define LINE_4           (1<<24)
-#define TRIANGLE_4       (1<<25)
-#define QUADRANGLE_4     (1<<26)
-#define TETRAHEDRON_4    (1<<27)
-#define HEXAHEDRON_4     (1<<28)
-#define PRISM_4          (1<<29)
-#define PYRAMID_4        (1<<30)
+#define LINE_3           (1<<19)
+#define TRIANGLE_3       (1<<20)
+#define QUADRANGLE_3     (1<<21)
+#define TETRAHEDRON_3    (1<<22)
+#define HEXAHEDRON_3     (1<<23)
+#define PRISM_3          (1<<24)
+#define PYRAMID_3        (1<<25)
+
+#define LINE_4           (1<<26)
+#define TRIANGLE_4       (1<<27)
+#define QUADRANGLE_4     (1<<28)
+#define TETRAHEDRON_4    (1<<29)
+#define HEXAHEDRON_4     (1<<30)
+#define PRISM_4          (1<<31)
+// #define PYRAMID_4        (1<<32)
 
 /* Adapt.Type */
 #define P1 1
