@@ -132,16 +132,19 @@ struct StringXDefine  Element_Type[] = {
   {"Tetrahedron4"   , TETRAHEDRON_4},
   {"Hexahedron"     , HEXAHEDRON},
   {"Hexahedron2"    , HEXAHEDRON_2},
+  {"Hexahedron2_20N", HEXAHEDRON_2_20N},
   {"Hexahedron3"    , HEXAHEDRON_3},
   {"Hexahedron4"    , HEXAHEDRON_4},
   {"Prism"          , PRISM},
   {"Prism2"         , PRISM_2},
+  {"Prism2_15N"     , PRISM_2_15N},
   {"Prism3"         , PRISM_3},
   {"Prism4"         , PRISM_4},
   {"Pyramid"        , PYRAMID},
   {"Pyramid2"       , PYRAMID_2},
+  {"Pyramid2_13N"   , PYRAMID_2_13N},
   {"Pyramid3"       , PYRAMID_3},
-  {"Pyramid4"       , PYRAMID_4},
+  //{"Pyramid4"       , PYRAMID_4},
   {NULL             , TRIANGLE}
  } ;
 
@@ -329,6 +332,7 @@ struct StringXPointer  Current_Value[] = {
   {"KSPIterations", &Current.KSPIterations}, {"KSPIts", &Current.KSPIterations},
   {"KSPIteration", &Current.KSPIteration},
   {"KSPResidual", &Current.KSPResidual},
+  {"KSPSystemSize", &Current.KSPSystemSize},
   {NULL       , NULL}
 } ;
 
@@ -430,16 +434,19 @@ struct DefineXFunction  FunctionForGauss[] = {
   {TETRAHEDRON_4  , (CAST)Gauss_Tetrahedron},
   {HEXAHEDRON     , (CAST)Gauss_Hexahedron},
   {HEXAHEDRON_2   , (CAST)Gauss_Hexahedron},
+  {HEXAHEDRON_2_20N, (CAST)Gauss_Hexahedron},
   {HEXAHEDRON_3   , (CAST)Gauss_Hexahedron},
   {HEXAHEDRON_4   , (CAST)Gauss_Hexahedron},
   {PRISM          , (CAST)Gauss_Prism},
   {PRISM_2        , (CAST)Gauss_Prism},
+  {PRISM_2_15N    , (CAST)Gauss_Prism},
   {PRISM_3        , (CAST)Gauss_Prism},
   {PRISM_4        , (CAST)Gauss_Prism},
   {PYRAMID        , (CAST)Gauss_Pyramid},
   {PYRAMID_2      , (CAST)Gauss_Pyramid},
+  {PYRAMID_2_13N  , (CAST)Gauss_Pyramid},
   {PYRAMID_3      , (CAST)Gauss_Pyramid},
-  {PYRAMID_4      , (CAST)Gauss_Pyramid},
+  //{PYRAMID_4      , (CAST)Gauss_Pyramid},
 #else
   {POINT          , NULL},
   {LINE           , NULL},
@@ -461,16 +468,19 @@ struct DefineXFunction  FunctionForGauss[] = {
   {TETRAHEDRON_4  , NULL},
   {HEXAHEDRON     , NULL},
   {HEXAHEDRON_2   , NULL},
+  {HEXAHEDRON_2_20N, NULL},
   {HEXAHEDRON_3   , NULL},
   {HEXAHEDRON_4   , NULL},
   {PRISM          , NULL},
   {PRISM_2        , NULL},
+  {PRISM_2_15N    , NULL},
   {PRISM_3        , NULL},
   {PRISM_4        , NULL},
   {PYRAMID        , NULL},
   {PYRAMID_2      , NULL},
+  {PYRAMID_2_13N  , NULL},
   {PYRAMID_3      , NULL},
-  {PYRAMID_4      , NULL},
+  //{PYRAMID_4      , NULL},
 #endif
   {0              , 0}
 } ;
@@ -510,9 +520,9 @@ struct DefineXFunction  FunctionForGaussLegendre[] = {
 #define TRI  TRIANGLE | TRIANGLE_2 | TRIANGLE_3 | TRIANGLE_4
 #define QUA  QUADRANGLE | QUADRANGLE_2 | QUADRANGLE_2_8N | QUADRANGLE_3 | QUADRANGLE_4
 #define TET  TETRAHEDRON | TETRAHEDRON_2 | TETRAHEDRON_3 | TETRAHEDRON_4
-#define HEX  HEXAHEDRON | HEXAHEDRON_2 | HEXAHEDRON_3 | HEXAHEDRON_4
-#define PRI  PRISM | PRISM_2 | PRISM_3 | PRISM_4
-#define PYR  PYRAMID | PYRAMID_2 | PYRAMID_3 | PYRAMID_4
+#define HEX  HEXAHEDRON | HEXAHEDRON_2 | HEXAHEDRON_2_20N | HEXAHEDRON_3 | HEXAHEDRON_4
+#define PRI  PRISM | PRISM_2 | PRISM_2_15N | PRISM_3 | PRISM_4
+#define PYR  PYRAMID | PYRAMID_2 | PYRAMID_2_13N | PYRAMID_3 // | PYRAMID_4
 #define ALL  POI|LIN|TRI|QUA|TET|HEX|PRI|PYR
 
 struct StringX3Function3Nbr  BF_Function[] = {
@@ -1123,6 +1133,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Memory"            , (CAST)F_GetMemory        ,   0,   0 },
   {"GetMemory"         , (CAST)F_GetMemory        ,   0,   0 },
   {"SetNumberRunTime"  , (CAST)F_SetNumberRunTime ,  -1,  -1 },
+  {"SetNumberRunTimeWithChoices"  , (CAST)F_SetNumberRunTimeWithChoices ,  -1,  -1 },
   {"GetNumberRunTime"  , (CAST)F_GetNumberRunTime ,  -1,  -1 },
   {"SetVariable"       , (CAST)F_SetVariable      ,  -1,  -1 },
   {"SetCumulativeVariable" , (CAST)F_SetCumulativeVariable ,  -1,  -1 },
