@@ -21,7 +21,7 @@
 
 #define SQU(a)     ((a)*(a))
 
-extern double Flag_ORDER ;
+extern double Flag_ORDER, Flag_MSH_SCALING ;
 
 FILE  * File_GEO ;
 char *name;
@@ -201,10 +201,13 @@ int Geo_GetNbNodesPerElement(int Type)
   case TRIANGLE_2    : return 6;
   case QUADRANGLE_2  : return 9;
   case TETRAHEDRON_2 : return 10;
-  case HEXAHEDRON_2  : return 20;
-  case PRISM_2       : return 15;
+  case HEXAHEDRON_2  : return 27;
+  case PRISM_2       : return 18;
   case PYRAMID_2     : return 13;
   case QUADRANGLE_2_8N: return 8;
+  case HEXAHEDRON_2_20N : return 20;
+  case PRISM_2_15N   : return 15;
+  case PYRAMID_2_13N : return 13;
 
   case LINE_3        : return 4;
   case TRIANGLE_3    : return 10;
@@ -220,7 +223,7 @@ int Geo_GetNbNodesPerElement(int Type)
   case TETRAHEDRON_4 : return 35;
   case HEXAHEDRON_4  : return 125;
   case PRISM_4       : return 75;
-  case PYRAMID_4     : return 55;
+  //case PYRAMID_4     : return 55;
 
   default :
     Message::Error("Unknown type of Element");
@@ -249,6 +252,9 @@ int Geo_GetDimOfElement(int Type)
   case PRISM_2       : return 3;
   case PYRAMID_2     : return 3;
   case QUADRANGLE_2_8N: return 2;
+  case HEXAHEDRON_2_20N : return 3;
+  case PRISM_2_15N   : return 3;
+  case PYRAMID_2_13N : return 3;
 
   case LINE_3        : return 1;
   case TRIANGLE_3    : return 2;
@@ -264,7 +270,7 @@ int Geo_GetDimOfElement(int Type)
   case TETRAHEDRON_4 : return 3;
   case HEXAHEDRON_4  : return 3;
   case PRISM_4       : return 3;
-  case PYRAMID_4     : return 3;
+  //case PYRAMID_4     : return 3;
   default :
     Message::Error("Unknown type of Element");
     return -1;
@@ -411,6 +417,11 @@ static void Geo_ReadFileWithGmsh(struct GeoData * GeoData_P)
     Geo_Node.x = coord[3*i + 0];
     Geo_Node.y = coord[3*i + 1];
     Geo_Node.z = coord[3*i + 2];
+    if(Flag_MSH_SCALING != 1.0){
+      Geo_Node.x *= Flag_MSH_SCALING;
+      Geo_Node.y *= Flag_MSH_SCALING;
+      Geo_Node.z *= Flag_MSH_SCALING;
+    }
     List_Add(GeoData_P->Nodes, &Geo_Node) ;
 
     if(!i){
@@ -666,6 +677,11 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
             if(fscanf(File_GEO, "%d %lf %lf %lf",
                       &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z) != 4)
               return;
+            if(Flag_MSH_SCALING != 1.0){
+              Geo_Node.x *= Flag_MSH_SCALING;
+              Geo_Node.y *= Flag_MSH_SCALING;
+              Geo_Node.z *= Flag_MSH_SCALING;
+            }
           }
           else {
             if(fread(&Geo_Node.Num, sizeof(int), 1, File_GEO) != 1) return;
@@ -676,6 +692,11 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
             Geo_Node.x = xyz[0];
             Geo_Node.y = xyz[1];
             Geo_Node.z = xyz[2];
+            if(Flag_MSH_SCALING != 1.0){
+              Geo_Node.x *= Flag_MSH_SCALING;
+              Geo_Node.y *= Flag_MSH_SCALING;
+              Geo_Node.z *= Flag_MSH_SCALING;
+            }
           }
         }
         else{
@@ -685,11 +706,21 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
               if(fscanf(File_GEO, "%d %lf %lf %lf %d %d",
                         &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z,
                         &dim, &entity) != 6) return;
+              if(Flag_MSH_SCALING != 1.0){
+                Geo_Node.x *= Flag_MSH_SCALING;
+                Geo_Node.y *= Flag_MSH_SCALING;
+                Geo_Node.z *= Flag_MSH_SCALING;
+              }
             }
             else{
               if(fscanf(File_GEO, "%d %lf %lf %lf %d",
                         &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z,
                         &entity) != 5) return;
+              if(Flag_MSH_SCALING != 1.0){
+                Geo_Node.x *= Flag_MSH_SCALING;
+                Geo_Node.y *= Flag_MSH_SCALING;
+                Geo_Node.z *= Flag_MSH_SCALING;
+              }
               if(entity){
                 if(fscanf(File_GEO, "%d", &dim) != 1) return;
               }
@@ -718,6 +749,11 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
             Geo_Node.x = xyz[0];
             Geo_Node.y = xyz[1];
             Geo_Node.z = xyz[2];
+            if(Flag_MSH_SCALING != 1.0){
+              Geo_Node.x *= Flag_MSH_SCALING;
+              Geo_Node.y *= Flag_MSH_SCALING;
+              Geo_Node.z *= Flag_MSH_SCALING;
+            }
           }
           if(dim == 1 && (Version < 3.0 || entity)){
             double u[1];
