@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 
 #include <string.h>
 #include "GeoData.h"
@@ -22,6 +22,7 @@ double * Geo_GetNodes_uvw(int Type, int *nbn)
 {
   switch(Type){
   case POINT       : *nbn = NbrNodes_Point ;       return(*Nodes_Point) ;
+
   case LINE        : *nbn = NbrNodes_Line ;        return(*Nodes_Line) ;
   case TRIANGLE    : *nbn = NbrNodes_Triangle ;    return(*Nodes_Triangle) ;
   case QUADRANGLE  : *nbn = NbrNodes_Quadrangle ;  return(*Nodes_Quadrangle) ;
@@ -29,11 +30,13 @@ double * Geo_GetNodes_uvw(int Type, int *nbn)
   case HEXAHEDRON  : *nbn = NbrNodes_Hexahedron ;  return(*Nodes_Hexahedron) ;
   case PRISM       : *nbn = NbrNodes_Prism ;       return(*Nodes_Prism) ;
   case PYRAMID     : *nbn = NbrNodes_Pyramid ;     return(*Nodes_Pyramid) ;
+
   case LINE_2      : *nbn = NbrNodes_Line_2 ;      return(*Nodes_Line_2) ;
   case TRIANGLE_2  : *nbn = NbrNodes_Triangle_2 ;  return(*Nodes_Triangle_2) ;
   case QUADRANGLE_2: *nbn = NbrNodes_Quadrangle_2 ;return(*Nodes_Quadrangle_2) ;
   case QUADRANGLE_2_8N: *nbn = NbrNodes_Quadrangle_2_8N ;return(*Nodes_Quadrangle_2_8N);
   case TETRAHEDRON_2: *nbn = NbrNodes_Tetrahedron_2; return(*Nodes_Tetrahedron_2) ;
+
   default :
     Message::Error("Unknown type of Element in Geo_GetNodes_uvw") ; return(NULL) ;
   }
@@ -105,19 +108,37 @@ int *Geo_GetIM_Den(int Type_Element, int * Nbe)
 {
 
   switch (Type_Element) {
-  case POINT :       *Nbe = NbrEdges_Point       ; return(NULL) ;
-  case LINE :        *Nbe = NbrEdges_Line        ; return(*Den_Line) ;
-  case TRIANGLE :    *Nbe = NbrEdges_Triangle    ; return(*Den_Triangle) ;
-  case QUADRANGLE :  *Nbe = NbrEdges_Quadrangle  ; return(*Den_Quadrangle) ;
-  case TETRAHEDRON : *Nbe = NbrEdges_Tetrahedron ; return(*Den_Tetrahedron) ;
-  case HEXAHEDRON :  *Nbe = NbrEdges_Hexahedron  ; return(*Den_Hexahedron) ;
-  case PRISM :       *Nbe = NbrEdges_Prism       ; return(*Den_Prism) ;
-  case PYRAMID :     *Nbe = NbrEdges_Pyramid     ; return(*Den_Pyramid) ;
-  case LINE_2 :      *Nbe = NbrEdges_Line_2      ; return(*Den_Line_2) ;
-  case TRIANGLE_2 :  *Nbe = NbrEdges_Triangle_2  ; return(*Den_Triangle_2) ;
-  case QUADRANGLE_2 :*Nbe = NbrEdges_Quadrangle_2; return(*Den_Quadrangle_2) ;
-  case QUADRANGLE_2_8N :*Nbe = NbrEdges_Quadrangle_2_8N; return(*Den_Quadrangle_2_8N) ;
-  case TETRAHEDRON_2 : *Nbe = NbrEdges_Tetrahedron_2; return(*Den_Tetrahedron_2) ;
+  case POINT :
+    *Nbe = NbrEdges_Point       ; return(NULL) ;
+
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+    *Nbe = NbrEdges_Line        ; return(*Den_Line) ;
+
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+    *Nbe = NbrEdges_Triangle    ; return(*Den_Triangle) ;
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
+    *Nbe = NbrEdges_Quadrangle  ; return(*Den_Quadrangle) ;
+
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+    *Nbe = NbrEdges_Tetrahedron ; return(*Den_Tetrahedron) ;
+
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
+    *Nbe = NbrEdges_Hexahedron  ; return(*Den_Hexahedron) ;
+
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+    *Nbe = NbrEdges_Prism       ; return(*Den_Prism) ;
+
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
+    *Nbe = NbrEdges_Pyramid     ; return(*Den_Pyramid) ;
+
   default :
     Message::Error("Unknown incidence matrix for element type %d", Type_Element);
     return(NULL) ;
@@ -131,19 +152,37 @@ int *Geo_GetIM_Den(int Type_Element, int * Nbe)
 int *Geo_GetIM_Dfe(int Type_Element, int * Nbf)
 {
   switch (Type_Element) {
-  case POINT :       *Nbf = NbrFacets_Point       ; return(NULL) ;
-  case LINE :        *Nbf = NbrFacets_Line        ; return(NULL) ;
-  case TRIANGLE :    *Nbf = NbrFacets_Triangle    ; return(*Dfe_Triangle) ;
-  case QUADRANGLE :  *Nbf = NbrFacets_Quadrangle  ; return(*Dfe_Quadrangle) ;
-  case TETRAHEDRON : *Nbf = NbrFacets_Tetrahedron ; return(*Dfe_Tetrahedron) ;
-  case HEXAHEDRON :  *Nbf = NbrFacets_Hexahedron  ; return(*Dfe_Hexahedron) ;
-  case PRISM :       *Nbf = NbrFacets_Prism       ; return(*Dfe_Prism) ;
-  case PYRAMID :     *Nbf = NbrFacets_Pyramid     ; return(*Dfe_Pyramid) ;
-  case LINE_2 :      *Nbf = NbrFacets_Line_2      ; return(NULL) ;
-  case TRIANGLE_2 :  *Nbf = NbrFacets_Triangle_2  ; return(*Dfe_Triangle_2) ;
-  case QUADRANGLE_2 :*Nbf = NbrFacets_Quadrangle_2; return(*Dfe_Quadrangle_2) ;
-  case QUADRANGLE_2_8N :*Nbf = NbrFacets_Quadrangle_2_8N; return(*Dfe_Quadrangle_2_8N) ;
-  case TETRAHEDRON_2 : *Nbf = NbrFacets_Tetrahedron_2; return(*Dfe_Tetrahedron_2);
+  case POINT :
+    *Nbf = NbrFacets_Point       ; return(NULL) ;
+
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+    *Nbf = NbrFacets_Line        ; return(NULL) ;
+
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+    *Nbf = NbrFacets_Triangle    ; return(*Dfe_Triangle) ;
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
+    *Nbf = NbrFacets_Quadrangle  ; return(*Dfe_Quadrangle) ;
+
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+     *Nbf = NbrFacets_Tetrahedron ; return(*Dfe_Tetrahedron) ;
+
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
+    *Nbf = NbrFacets_Hexahedron  ; return(*Dfe_Hexahedron) ;
+
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+    *Nbf = NbrFacets_Prism       ; return(*Dfe_Prism) ;
+
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
+    *Nbf = NbrFacets_Pyramid     ; return(*Dfe_Pyramid) ;
+
   default :
     Message::Error("Unknown incidence matrix for element type %d", Type_Element);
     return(NULL) ;
@@ -157,19 +196,37 @@ int *Geo_GetIM_Dfe(int Type_Element, int * Nbf)
 int *Geo_GetIM_Dfn(int Type_Element, int * Nbf)
 {
   switch (Type_Element) {
-  case POINT :      *Nbf = NbrFacets_Point       ; return(NULL) ;
-  case LINE :       *Nbf = NbrFacets_Line        ; return(NULL) ;
-  case TRIANGLE :   *Nbf = NbrFacets_Triangle    ; return(*Dfn_Triangle) ;
-  case QUADRANGLE : *Nbf = NbrFacets_Quadrangle  ; return(*Dfn_Quadrangle) ;
-  case TETRAHEDRON :*Nbf = NbrFacets_Tetrahedron ; return(*Dfn_Tetrahedron) ;
-  case HEXAHEDRON : *Nbf = NbrFacets_Hexahedron  ; return(*Dfn_Hexahedron) ;
-  case PRISM :	    *Nbf = NbrFacets_Prism       ; return(*Dfn_Prism) ;
-  case PYRAMID :    *Nbf = NbrFacets_Pyramid     ; return(*Dfn_Pyramid) ;
-  case LINE_2 :     *Nbf = NbrFacets_Line_2      ; return(NULL) ;
-  case TRIANGLE_2 : *Nbf = NbrFacets_Triangle_2  ; return(*Dfn_Triangle_2) ;
-  case QUADRANGLE_2:*Nbf = NbrFacets_Quadrangle_2; return(*Dfn_Quadrangle_2) ;
-  case QUADRANGLE_2_8N:*Nbf = NbrFacets_Quadrangle_2_8N; return(*Dfn_Quadrangle_2_8N) ;
-  case TETRAHEDRON_2: *Nbf = NbrFacets_Tetrahedron_2; return(*Dfn_Tetrahedron_2);
+  case POINT :
+    *Nbf = NbrFacets_Point       ; return(NULL) ;
+
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
+    *Nbf = NbrFacets_Line        ; return(NULL) ;
+
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
+    *Nbf = NbrFacets_Triangle    ; return(*Dfn_Triangle) ;
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
+    *Nbf = NbrFacets_Quadrangle  ; return(*Dfn_Quadrangle) ;
+
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
+    *Nbf = NbrFacets_Tetrahedron ; return(*Dfn_Tetrahedron) ;
+
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
+    *Nbf = NbrFacets_Hexahedron  ; return(*Dfn_Hexahedron) ;
+
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
+    *Nbf = NbrFacets_Prism       ; return(*Dfn_Prism) ;
+
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
+    *Nbf = NbrFacets_Pyramid     ; return(*Dfn_Pyramid) ;
+
   default :
     Message::Error("Unknown incidence matrix for element type %d", Type_Element);
     return(NULL) ;
@@ -186,42 +243,42 @@ int * Geo_GetIM_Den_Xp(int Type_Element, int * Nbe, int * Nbn)
   case POINT :
     *Nbe = NbrEdges_Point ; *Nbn = NbrNodes_Point ;
     return(NULL) ;
-  case LINE :
+
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
     *Nbe = NbrEdges_Line ; *Nbn = NbrNodes_Line ;
     return(Den_Line_Xp) ;
-  case TRIANGLE :
+
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
     *Nbe = NbrEdges_Triangle ; *Nbn = NbrNodes_Triangle ;
     return(Den_Triangle_Xp) ;
-  case QUADRANGLE :
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     *Nbe = NbrEdges_Quadrangle ; *Nbn = NbrNodes_Quadrangle ;
     return(Den_Quadrangle_Xp) ;
-  case TETRAHEDRON :
+
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     *Nbe = NbrEdges_Tetrahedron ; *Nbn = NbrNodes_Tetrahedron ;
     return(Den_Tetrahedron_Xp) ;
-  case HEXAHEDRON :
+
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     *Nbe = NbrEdges_Hexahedron ; *Nbn = NbrNodes_Hexahedron ;
     return(Den_Hexahedron_Xp) ;
-  case PRISM :
+
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
     *Nbe = NbrEdges_Prism ; *Nbn = NbrNodes_Prism ;
     return(Den_Prism_Xp) ;
-  case PYRAMID :
+
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     *Nbe = NbrEdges_Pyramid ; *Nbn = NbrNodes_Pyramid ;
     return(Den_Pyramid_Xp) ;
-  case LINE_2 :
-    *Nbe = NbrEdges_Line_2 ; *Nbn = NbrNodes_Line_2 ;
-    return(Den_Line_2_Xp) ;
-  case TRIANGLE_2 :
-    *Nbe = NbrEdges_Triangle_2 ; *Nbn = NbrNodes_Triangle_2 ;
-    return(Den_Triangle_2_Xp) ;
-  case QUADRANGLE_2 :
-    *Nbe = NbrEdges_Quadrangle_2 ; *Nbn = NbrNodes_Quadrangle_2 ;
-    return(Den_Quadrangle_2_Xp) ;
-  case QUADRANGLE_2_8N :
-    *Nbe = NbrEdges_Quadrangle_2_8N ; *Nbn = NbrNodes_Quadrangle_2_8N ;
-    return(Den_Quadrangle_2_8N_Xp) ;
-  case TETRAHEDRON_2 :
-    *Nbe = NbrEdges_Tetrahedron_2 ; *Nbn = NbrNodes_Tetrahedron_2 ;
-    return(Den_Tetrahedron_2_Xp) ;
+
   default :
     Message::Error("Unknown incidence matrix for element type %d", Type_Element);
     return(NULL) ;
@@ -238,42 +295,42 @@ int * Geo_GetIM_Dfe_Xp(int Type_Element, int * nbf, int * nbe)
   case POINT :
     *nbf = NbrFacets_Point ; *nbe = NbrEdges_Point ;
     return(NULL) ;
-  case LINE :
+
+  case LINE   : case LINE_2 :
+  case LINE_3 : case LINE_4 :
     *nbf = NbrFacets_Line ; *nbe = NbrEdges_Line ;
     return(NULL) ;
-  case TRIANGLE :
+
+  case TRIANGLE   : case TRIANGLE_2 :
+  case TRIANGLE_3 : case TRIANGLE_4 :
     *nbf = NbrFacets_Triangle ; *nbe = NbrEdges_Triangle ;
     return(Dfe_Triangle_Xp) ;
-  case QUADRANGLE :
+
+  case QUADRANGLE   : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+  case QUADRANGLE_3 : case QUADRANGLE_4 :
     *nbf = NbrFacets_Quadrangle ; *nbe = NbrEdges_Quadrangle ;
     return(Dfe_Quadrangle_Xp) ;
-  case TETRAHEDRON :
+
+  case TETRAHEDRON   : case TETRAHEDRON_2 :
+  case TETRAHEDRON_3 : case TETRAHEDRON_4 :
     *nbf = NbrFacets_Tetrahedron ; *nbe = NbrEdges_Tetrahedron ;
     return(Dfe_Tetrahedron_Xp) ;
-  case HEXAHEDRON :
+
+  case HEXAHEDRON   : case HEXAHEDRON_2 : case HEXAHEDRON_2_20N :
+  case HEXAHEDRON_3 : case HEXAHEDRON_4 :
     *nbf = NbrFacets_Hexahedron ; *nbe = NbrEdges_Hexahedron ;
     return(Dfe_Hexahedron_Xp) ;
-  case PRISM :
+
+  case PRISM   : case PRISM_2 : case PRISM_2_15N :
+  case PRISM_3 : case PRISM_4 :
     *nbf = NbrFacets_Prism ; *nbe = NbrEdges_Prism ;
     return(Dfe_Prism_Xp) ;
-  case PYRAMID :
+
+  case PYRAMID : case PYRAMID_2 : case PYRAMID_2_13N :
+  case PYRAMID_3 : // case PYRAMID_4
     *nbf = NbrFacets_Pyramid ; *nbe = NbrEdges_Pyramid ;
     return(Dfe_Pyramid_Xp) ;
-  case LINE_2 :
-    *nbf = NbrFacets_Line_2 ; *nbe = NbrEdges_Line_2 ;
-    return(NULL) ;
-  case TRIANGLE_2 :
-    *nbf = NbrFacets_Triangle_2 ; *nbe = NbrEdges_Triangle_2 ;
-    return(Dfe_Triangle_2_Xp) ;
-  case QUADRANGLE_2 :
-    *nbf = NbrFacets_Quadrangle_2 ; *nbe = NbrEdges_Quadrangle_2 ;
-    return(Dfe_Quadrangle_2_Xp) ;
-  case QUADRANGLE_2_8N :
-    *nbf = NbrFacets_Quadrangle_2_8N ; *nbe = NbrEdges_Quadrangle_2_8N ;
-    return(Dfe_Quadrangle_2_8N_Xp) ;
-  case TETRAHEDRON_2 :
-    *nbf = NbrFacets_Tetrahedron_2 ; *nbe = NbrEdges_Tetrahedron_2 ;
-    return(Dfe_Tetrahedron_2_Xp) ;
+
   default :
     Message::Error("Unknown incidence matrix for element type %d", Type_Element);
     return(NULL) ;
@@ -495,8 +552,8 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
   while (1) {
 
     do {
-      fgets(String, sizeof(String), File_PRE) ;
-      if (feof(File_PRE))  break ;
+      if(!fgets(String, sizeof(String), File_PRE) || feof(File_PRE))
+        break ;
     } while (String[0] != '$') ;
 
     if (feof(File_PRE))  break ;
@@ -505,7 +562,10 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
 
     if (!strncmp(&String[1], "ElementsXEdges", 14)) {
 
-      fscanf(File_PRE, "%d", &GeoDataIndex) ;
+      if(fscanf(File_PRE, "%d", &GeoDataIndex) != 1){
+        Message::Error("Could not read GeoData index");
+        return;
+      }
       if(GeoDataIndex > NbrGeoData-1){
 	Message::Error("Unknown GeoData: %d", GeoDataIndex);
         return;
@@ -514,14 +574,24 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
       GeoData_P = GeoData_P0 + GeoDataIndex ;
       Geo_Element_P0 = (struct Geo_Element*)List_Pointer(GeoData_P->Elements, 0) ;
 
-      fscanf(File_PRE, "%d", &GeoData_P->NbrElementsWithEdges) ;
+      if(fscanf(File_PRE, "%d", &GeoData_P->NbrElementsWithEdges) != 1){
+        Message::Error("Could not read number of elements with edges");
+        return;
+      }
       for (i = 0 ; i < GeoData_P->NbrElementsWithEdges ; i++) {
-	fscanf(File_PRE, "%d %d", &Index_Element, &Nbr_Entities) ;
+	if(fscanf(File_PRE, "%d %d", &Index_Element, &Nbr_Entities) != 2){
+          Message::Error("Could not read element index and number of edges");
+          return;
+        }
 	Geo_Element_P = Geo_Element_P0 + Index_Element ;
 	Geo_Element_P->NbrEdges = Nbr_Entities ;
 	Geo_Element_P->NumEdges = (int *)Malloc(Nbr_Entities * sizeof(int)) ;
-	for (j = 0 ; j < Geo_Element_P->NbrEdges ; j++)
-	  fscanf(File_PRE, "%d", &Geo_Element_P->NumEdges[j]) ;
+	for (j = 0 ; j < Geo_Element_P->NbrEdges ; j++){
+	  if(fscanf(File_PRE, "%d", &Geo_Element_P->NumEdges[j]) != 1){
+            Message::Error("Could not read edge");
+            return;
+          }
+        }
       }
     }
 
@@ -529,7 +599,10 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
 
     else if (!strncmp(&String[1], "ElementsXFacets", 15)) {
 
-      fscanf(File_PRE, "%d", &GeoDataIndex) ;
+      if(fscanf(File_PRE, "%d", &GeoDataIndex) != 1){
+        Message::Error("Could not read GeoData index");
+        return;
+      }
       if(GeoDataIndex > NbrGeoData-1){
 	Message::Error("Unknown GeoData: %d", GeoDataIndex);
         return;
@@ -538,14 +611,24 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
       GeoData_P = GeoData_P0 + GeoDataIndex ;
       Geo_Element_P0 = (struct Geo_Element*)List_Pointer(GeoData_P->Elements, 0) ;
 
-      fscanf(File_PRE, "%d", &GeoData_P->NbrElementsWithFacets) ;
+      if(fscanf(File_PRE, "%d", &GeoData_P->NbrElementsWithFacets) != 1){
+        Message::Error("Could not read number of elements with facets");
+        return;
+      }
       for (i = 0 ; i < GeoData_P->NbrElementsWithFacets ; i++) {
-	fscanf(File_PRE, "%d %d", &Index_Element, &Nbr_Entities) ;
+	if(fscanf(File_PRE, "%d %d", &Index_Element, &Nbr_Entities) != 2){
+          Message::Error("Could not read element index and number of facets");
+          return;
+        }
 	Geo_Element_P = Geo_Element_P0 + Index_Element ;
 	Geo_Element_P->NbrFacets = Nbr_Entities ;
 	Geo_Element_P->NumFacets = (int *)Malloc(Nbr_Entities * sizeof(int)) ;
-	for (j = 0 ; j < Geo_Element_P->NbrFacets ; j++)
-	  fscanf(File_PRE, "%d", &Geo_Element_P->NumFacets[j]) ;
+	for (j = 0 ; j < Geo_Element_P->NbrFacets ; j++){
+	  if(fscanf(File_PRE, "%d", &Geo_Element_P->NumFacets[j]) != 1){
+            Message::Error("Could not read facet");
+            return;
+          }
+        }
       }
     }
 
@@ -553,18 +636,24 @@ void Geo_ReadFilePRE(struct GeoData * GeoData_P0, int NbrGeoData,
 
     else if (!strncmp(&String[1], "ExtendedGroup", 13)) {
 
-      fscanf(File_PRE, "%d %d", &Index_Group, &Nbr_Entities) ;
+      if(fscanf(File_PRE, "%d %d", &Index_Group, &Nbr_Entities) != 2){
+        Message::Error("Could not read extended group index and number of entities");
+        return;
+      }
       Group_P = (struct Group*)List_Pointer(Group_L, Index_Group) ;
       Group_P->ExtendedList = List_Create(Nbr_Entities, 1, sizeof(int)) ;
       for (i = 0 ; i < Nbr_Entities ; i++) {
-	fscanf(File_PRE, "%d", &Num_Entity) ;
+	if(fscanf(File_PRE, "%d", &Num_Entity) != 1){
+          Message::Error("Could not read entity");
+          return;
+        }
 	List_Add(Group_P->ExtendedList, &Num_Entity) ;
       }
     }
 
     do {
-      fgets(String, sizeof(String), File_PRE) ;
-      if (feof(File_PRE)){ Message::Error("Prematured end of file"); return; }
+      if(!fgets(String, sizeof(String), File_PRE) || feof(File_PRE))
+        break;
     } while (String[0] != '$') ;
 
   }   /* while 1 ... */
