@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 //
 // Contributor(s):
 //   David Colignon
@@ -117,14 +117,6 @@ void LinAlg_InitializeSolver(int* argc, char*** argv)
 
 void LinAlg_FinalizeSolver()
 {
-  // this causes random crashes with PETSC 3 when doing several
-  // initialize/finalize calls (when using getdp as a library). Until we figure
-  // out what's happening, let's simply initialize petsc/slepc once, and never
-  // finalize.
-#if (PETSC_VERSION_MAJOR == 3)  && (PETSC_VERSION_MINOR <= 7)
-  return;
-#endif
-
   if(SolverInitialized){
 #if defined(HAVE_SLEPC)
     SlepcFinalize();
@@ -274,11 +266,23 @@ void LinAlg_DestroySolver(gSolver *Solver)
 {
   for(int i = 0; i < 10; i++){
 #if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)
-    if(Solver->ksp[i]) _try(KSPDestroy(&Solver->ksp[i]));
-    if(Solver->snes[i]) _try(SNESDestroy(&Solver->snes[i]));
+    if(Solver->ksp[i]){
+      _try(KSPDestroy(&Solver->ksp[i]));
+      Solver->ksp[i] = NULL;
+    }
+    if(Solver->snes[i]){
+      _try(SNESDestroy(&Solver->snes[i]));
+      Solver->snes[i] = NULL;
+    }
 #else
-    if(Solver->ksp[i]) _try(KSPDestroy(Solver->ksp[i]));
-    if(Solver->snes[i]) _try(SNESDestroy(Solver->snes[i]));
+    if(Solver->ksp[i]){
+      _try(KSPDestroy(Solver->ksp[i]));
+      Solver->ksp[i] = NULL;
+    }
+    if(Solver->snes[i]){
+      _try(SNESDestroy(Solver->snes[i]));
+      Solver->snes[i] = NULL;
+    }
 #endif
   }
 }
@@ -393,7 +397,10 @@ void LinAlg_ReadVector(FILE *file, gVector *V)
   PetscInt n;
   _try(VecGetSize(V->V, &n));
   PetscScalar *tmp = (PetscScalar*)Malloc(n*sizeof(PetscScalar));
-  fread(tmp, sizeof(PetscScalar), n, file);
+  if(!fread(tmp, sizeof(PetscScalar), n, file)){
+    Message::Error("Could not read vector");
+    return;
+  }
   for(PetscInt i = 0; i < n; i++){
     _try(VecSetValues(V->V, 1, &i, &tmp[i], INSERT_VALUES));
   }
@@ -931,6 +938,14 @@ void LinAlg_AddVectorProdVectorDouble(gVector *V1, gVector *V2, double d, gVecto
     Message::Error("Wrong arguments in 'LinAlg_AddVectorProdVectorDouble'");
 }
 
+void LinAlg_AddProdVectorDoubleProdVectorDouble(double alpha, gVector *V1,
+        double beta, gVector *V2, gVector *V3)
+{
+  PetscScalar alpha1 = alpha, beta1 = beta;
+  PetscScalar gamma1 = 0.0;
+  _try(VecAXPBYPCZ(V3->V, alpha1, beta1, gamma1, V1->V, V2->V));
+}
+
 void LinAlg_AddMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
 {
   PetscScalar tmp = 1.0;
@@ -1159,6 +1174,8 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
   PetscInt n;
   _try(VecGetLocalSize(B->V, &n));
 
+  Current.KSPSystemSize = n;
+
   int *row = (int*)Malloc(nnz * sizeof(int));
   int *col = (int*)Malloc(nnz * sizeof(int));
   double *valr = (double*)Malloc(nnz * sizeof(double));
@@ -1268,6 +1285,7 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
 
   PetscInt i, j;
   _try(MatGetSize(A->M, &i, &j));
+  Current.KSPSystemSize = i;
   if(!i){
     Message::Warning("Zero-size system: skipping solve!");
     return;
@@ -1368,6 +1386,14 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
     if(its > 1) Message::Info("%d iterations", its);
   }
   Current.KSPIterations = its;
+
+  PetscTruth set, kspfree = PETSC_FALSE;
+  PetscOptionsGetTruth(PETSC_NULL, "-kspfree", &kspfree, &set);
+  if(kspfree){
+    Message::Info("Freeing KSP solver");
+    LinAlg_DestroySolver(Solver);
+  }
+
 }
 
 void LinAlg_Solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
