@@ -30,7 +30,7 @@
 int     Flag_PRE = 0, Flag_CAL = 0, Flag_POS = 0, Flag_RESTART = 0;
 int     Flag_XDATA = 0, Flag_BIN = 0, Flag_SPLIT = 0, Flag_GMSH_VERSION = 1;
 int     Flag_NETWORK_CACHE = 0, Flag_CALLED_WITH_ONELAB_SERVER = 0;
-double  Flag_ORDER = -1.;
+double  Flag_ORDER = -1., Flag_MSH_SCALING = 1.;
 char   *Name_Generic = 0, *Name_Path = 0;
 char   *Name_Resolution = 0;
 char   *Name_MshFile = 0, *Name_AdaptFile = 0;
@@ -52,6 +52,7 @@ static void Info(int level, char *arg0)
 	    "  -cal                      processing\n"
 	    "  -pos 'PostOperation(s)'   post-processing\n"
 	    "  -msh file                 read mesh (in msh format) from file\n"
+            "  -msh_scaling value        scale the input mesh by the given value\n"
             "  -gmshread file(s)         read gmsh data (same as GmshRead in resolution)\n"
             "  -gmshtag tag(s)           tag(s) associated to GmshRead data\n"
 	    "  -restart                  resume processing from where it stopped\n"
@@ -355,6 +356,16 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
 	}
       }
 
+      else if (!strcmp(argv[i]+1, "msh_scaling")) {
+	i++;
+	if (i < argc && argv[i][0] != '-') {
+	  Flag_MSH_SCALING = atof(argv[i]); i++;
+	}
+	else {
+	  Message::Error("Missing scaling value");
+	}
+      }
+
       else if (!strcmp(argv[i]+1, "adapt") ||
 	       !strcmp(argv[i]+1, "adap") ||
 	       !strcmp(argv[i]+1, "ada")) {
@@ -488,6 +499,8 @@ public:
       Message::Error("%s", msg.c_str());
     else if(level == "Warning")
       Message::Warning("%s", msg.c_str());
+    else if(level == "Progress"){
+    }
     else
       Message::Info("%s", msg.c_str());
   }
@@ -702,7 +715,11 @@ int MainKernel(int argc, char *argv[])
   if(Flag_PRE || Flag_CAL || Flag_POS)
     SolvingAnalyse();
 
-  LinAlg_FinalizeSolver();
+  // PETSc cannot be finalized if it will be re-initialized again in the same
+  // process - so just don't finalize it if we use getdp as a library with a
+  // provided onelab server (e.g. for the mobile apps)
+  if(!Flag_CALLED_WITH_ONELAB_SERVER)
+    LinAlg_FinalizeSolver();
 
   Message::PrintErrorCounter("Run");
   Message::Cpu(3, true, true, true, true, true, "Stopped");
@@ -729,6 +746,7 @@ int getdp(const std::vector<std::string> &args, void *ptr)
   if(onelabServer != NULL){
     onelab::server::setInstance(onelabServer);
     Flag_CALLED_WITH_ONELAB_SERVER = 1;
+    Message::SetExitOnError(2); // throw exception on error
   }
   int argc = args.size();
   std::vector<char*> argv(argc + 1, (char*)0);
