@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 //
 // Contributed by Bertrand Thierry
 
@@ -917,13 +917,20 @@ static PetscErrorCode PrintVec(Vec b, const char* filename, const char* varname)
 static PetscErrorCode Jacobi_Solver(Mat A, Vec X, Vec B, double Tol, int MaxIter)
 {
   Vec X_old, W;
-  double residu;
+  double residu, residuInit;
 
   _try(VecSet(X, 0.));
   _try(VecDuplicate(X, &X_old));
   _try(VecDuplicate(X, &W));
   _try(VecCopy(X, W));
 
+  _try(VecNorm(B, NORM_2, &residuInit));
+  Message::Info(3, "Jacobi initial residual %g", residuInit);
+  Current.Iteration = 0;
+  Current.KSPIteration = 0;
+  Current.Residual = residuInit;
+  Current.KSPResidual = residuInit;
+
   for (int j=1; j < MaxIter; j++){
     _try(VecCopy(X, X_old));
     _try(MatMultILSMat(A, X_old, X));
@@ -932,7 +939,12 @@ static PetscErrorCode Jacobi_Solver(Mat A, Vec X, Vec B, double Tol, int MaxIter
     _try(VecWAXPY(W, -1.,X_old, X)); //W = X-X_old
     _try(VecNorm(W, NORM_2, &residu));
     Message::Info(3, "Jacobi iteration %d residual %g", j, residu);
-    if(residu < Tol) break;
+
+    Current.Iteration = j;
+    Current.KSPIteration = j;
+    Current.Residual = residu;
+    Current.KSPResidual = residu;
+    if(residu/residuInit < Tol) break;
   }
   PetscFunctionReturn(0);
 }
@@ -1239,10 +1251,19 @@ int Operation_BroadcastFields(struct Resolution  *Resolution_P,
                               struct DofData     *DofData_P0,
                               struct GeoData     *GeoData_P0)
 {
+  if(!MyStaticField || !AllStaticField){
+    // we are not in Operation_IterativeLinearSolver: call the new, generic
+    // BroadcastFields operation:
+    return Operation_BroadcastFieldsGeneric(Resolution_P, Operation_P,
+                                            DofData_P0, GeoData_P0);
+  }
+
+  // in the IterativeLinearSolver-specific BroadCastFields operation, the list
+  // of view tags contains the list of views to *not* broadcast
   std::set<int> fieldsToSkip;
-  for(int i = 0; i < List_Nbr(Operation_P->Case.BroadcastFields.FieldsToSkip); i++){
+  for(int i = 0; i < List_Nbr(Operation_P->Case.BroadcastFields.ViewTags); i++){
     double j;
-    List_Read(Operation_P->Case.BroadcastFields.FieldsToSkip, i, &j);
+    List_Read(Operation_P->Case.BroadcastFields.ViewTags, i, &j);
     fieldsToSkip.insert((int) j);
   }
 
@@ -1266,8 +1287,8 @@ int Operation_BroadcastFields(struct Resolution  *Resolution_P,
                               struct DofData     *DofData_P0,
                               struct GeoData     *GeoData_P0)
 {
-  Message::Error("BroadcastFields requires PETSc and Gmsh");
-  return 0;
+  return Operation_BroadcastFieldsGeneric(Resolution_P, Operation_P,
+                                          DofData_P0, GeoData_P0);
 }
 
 #endif
