@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2019 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues.
 
 #include <string.h>
 #include <math.h>
@@ -443,7 +443,8 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
   // force Gmsh version 1 for anything else than OnElementsOf, or if we store in
   // memory (which requires old-style list ordering)
   int oldVersion = Flag_GMSH_VERSION;
-  if(PostSubOperation_P->SubType != PRINT_ONELEMENTSOF ||
+  if(PostSubOperation_P->Type != POP_PRINT ||
+     PostSubOperation_P->SubType != PRINT_ONELEMENTSOF ||
      PostSubOperation_P->Depth != 1 ||
      PostSubOperation_P->StoreInField >= 0)
     Flag_GMSH_VERSION = 1;
@@ -546,7 +547,7 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
 
       std::vector<GEntity*> entities;
       m->getEntities(entities);
-      std::map<MVertex*, std::vector<double>, MVertexLessThanNum> newcoords;
+      std::map<MVertex*, std::vector<double>, MVertexPtrLessThan> newcoords;
       for(unsigned int i = 0; i < entities.size(); i++) {
         for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++) {
           MVertex* v = entities[i]->mesh_vertices[j];
@@ -558,7 +559,7 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
         }
       }
 
-      for(std::map<MVertex*, std::vector<double>, MVertexLessThanNum>::iterator
+      for(std::map<MVertex*, std::vector<double>, MVertexPtrLessThan>::iterator
             it = newcoords.begin(); it != newcoords.end(); it++) {
         it->first->x() = it->second[0];
         it->first->y() = it->second[1];
